import pyperclip
import logging
from flask import Blueprint, request, jsonify
from ..plugin import LocalBridgePlugin
import threading

class ClipboardPlugin(LocalBridgePlugin):
    def __init__(self, history_size=50):
        super().__init__(
            name="clipboard",
            uri="local://clipboard",
            methods=["GET", "POST"],
            description="Access system clipboard and clipboard history"
        )
        self.blueprint = Blueprint("clipboard", __name__)
        self.logger = logging.getLogger("LocalBridgeClipboardPlugin")
        self.history = []
        self.history_size = history_size
        self._lock = threading.Lock()

    def register(self, app, config):
        @self.blueprint.route("/localbridge/clipboard", methods=["GET"])
        def get_clipboard():
            if not self._check_token(request, config.token):
                return jsonify({"error": "Unauthorized"}), 401
            try:
                clipboard = pyperclip.paste()
                self._add_to_history(clipboard)
                self.logger.info("Clipboard read successfully.")
                return jsonify({"clipboard": clipboard})
            except Exception as e:
                self.logger.error(f"Failed to read clipboard: {e}")
                return jsonify({"error": str(e)}), 500

        @self.blueprint.route("/localbridge/clipboard", methods=["POST"])
        def set_clipboard():
            if not self._check_token(request, config.token):
                return jsonify({"error": "Unauthorized"}), 401
            data = request.json
            try:
                text = data.get("text", "")
                pyperclip.copy(text)
                self._add_to_history(text)
                self.logger.info("Clipboard set successfully.")
                return jsonify({"success": True})
            except Exception as e:
                self.logger.error(f"Failed to set clipboard: {e}")
                return jsonify({"success": False, "error": str(e)}), 500

        @self.blueprint.route("/localbridge/clipboard/history", methods=["GET"])
        def get_clipboard_history():
            if not self._check_token(request, config.token):
                return jsonify({"error": "Unauthorized"}), 401
            with self._lock:
                return jsonify({"history": list(self.history)})

        app.register_blueprint(self.blueprint)

    def _add_to_history(self, text):
        if not text:
            return
        with self._lock:
            if not self.history or self.history[-1] != text:
                self.history.append(text)
                if len(self.history) > self.history_size:
                    self.history.pop(0)

    def _check_token(self, req, token):
        return req.headers.get("X-LocalBridge-Agent-Token") == token
