import os
from ..plugin import LocalBridgePlugin
from flask import Blueprint, request, jsonify, send_file
import logging

class FileSystemPlugin(LocalBridgePlugin):
    def __init__(self):
        super().__init__(
            name="filesystem",
            uri="local://file",
            methods=["GET", "POST", "DELETE"],
            description="Access and manage local files and directories"
        )
        self.blueprint = Blueprint("filesystem", __name__)
        self.logger = logging.getLogger("LocalBridgeFileSystemPlugin")

    def register(self, app, config):
        @self.blueprint.route("/localbridge/file/<path:filepath>", methods=["GET"])
        def get_file(filepath):
            if not self._check_token(request, config.token):
                return jsonify({"error": "Unauthorized"}), 401
            abs_path = os.path.abspath(filepath)
            if not os.path.exists(abs_path):
                return jsonify({"error": "File not found"}), 404
            try:
                # For text files, return content; for binary, send as file
                if os.path.isdir(abs_path):
                    return jsonify({"directory": abs_path, "files": os.listdir(abs_path)})
                with open(abs_path, "rb") as f:
                    content = f.read()
                try:
                    # Try to decode as text
                    return jsonify({"file": abs_path, "content": content.decode("utf-8")})
                except Exception:
                    # If binary, send as file
                    return send_file(abs_path, as_attachment=True)
            except Exception as e:
                self.logger.error(f"Failed to read file: {e}")
                return jsonify({"error": str(e)}), 500

        @self.blueprint.route("/localbridge/file/<path:filepath>", methods=["POST"])
        def write_file(filepath):
            if not self._check_token(request, config.token):
                return jsonify({"error": "Unauthorized"}), 401
            abs_path = os.path.abspath(filepath)
            data = request.json
            try:
                content = data.get("content", "")
                with open(abs_path, "w", encoding="utf-8") as f:
                    f.write(content)
                self.logger.info(f"Wrote to file: {abs_path}")
                return jsonify({"success": True, "file": abs_path})
            except Exception as e:
                self.logger.error(f"Failed to write file: {e}")
                return jsonify({"success": False, "error": str(e)}), 500

        @self.blueprint.route("/localbridge/file/<path:filepath>", methods=["DELETE"])
        def delete_file(filepath):
            if not self._check_token(request, config.token):
                return jsonify({"error": "Unauthorized"}), 401
            abs_path = os.path.abspath(filepath)
            try:
                if os.path.isdir(abs_path):
                    os.rmdir(abs_path)
                else:
                    os.remove(abs_path)
                self.logger.info(f"Deleted: {abs_path}")
                return jsonify({"success": True, "file": abs_path})
            except Exception as e:
                self.logger.error(f"Failed to delete file: {e}")
                return jsonify({"success": False, "error": str(e)}), 500

        app.register_blueprint(self.blueprint)

    def _check_token(self, req, token):
        return req.headers.get("X-LocalBridge-Agent-Token") == token
