from ..plugin import LocalBridgePlugin
from flask import Blueprint, request, jsonify
import logging
import platform
import subprocess

class NotificationPlugin(LocalBridgePlugin):
    def __init__(self):
        super().__init__(
            name="notification",
            uri="local://notification",
            methods=["POST"],
            description="Send a native desktop notification"
        )
        self.blueprint = Blueprint("notification", __name__)
        self.logger = logging.getLogger("LocalBridgeNotificationPlugin")

    def register(self, app, config):
        @self.blueprint.route("/localbridge/notification", methods=["POST"])
        def send_notification():
            if not self._check_token(request, config.token):
                return jsonify({"error": "Unauthorized"}), 401
            data = request.json
            title = data.get("title", "Notification")
            message = data.get("message", "")
            try:
                self._send_notification(title, message)
                self.logger.info(f"Notification sent: {title} - {message}")
                return jsonify({"success": True})
            except Exception as e:
                self.logger.error(f"Failed to send notification: {e}")
                return jsonify({"success": False, "error": str(e)}), 500

        app.register_blueprint(self.blueprint)

    def _send_notification(self, title, message):
        system = platform.system()
        if system == "Darwin":  # macOS
            # Use AppleScript to send notification
            script = f'display notification "{message}" with title "{title}"'
            subprocess.run(["osascript", "-e", script], check=True)
        elif system == "Linux":
            # Use notify-send
            subprocess.run(["notify-send", title, message], check=True)
        elif system == "Windows":
            # Use ToastNotifier from win10toast (requires pip install win10toast)
            try:
                from win10toast import ToastNotifier
                toaster = ToastNotifier()
                toaster.show_toast(title, message, duration=5)
            except ImportError:
                raise RuntimeError("win10toast is not installed")
        else:
            raise RuntimeError(f"Unsupported OS: {system}")

    def _check_token(self, req, token):
        return req.headers.get("X-LocalBridge-Agent-Token") == token
